#!/usr/bin/perl -w
package ContextWindow;

#Erik Velldal, 2003, erikve@ilf.uio.no

#takes one ore more files or directories as arguments and (recursively)
#prints content word context of words in a window of size -n, where -n
#denotes number of sentence (span / window width) and defaults to 3. 
#"a word" is here taken to be its first lemma form.
#only files with the extension .syn are processed. prints to terminal
#(or STDOUT) if -t is set. if -t switch is not set, new files are
#created with the '.syn'-suffix replaced by the extension '.wic'.
# 
#specifying a value for the option  -i initiates the sentence counter,
#default is 0 (nb: this means first sentence's id = 1). useful when
#working with a corpus that is spread out over several files that are
#processed separately.   
#
#output format (tab separated):
#focus-word  context-word-1  ...  context-word-n  sentence-id 

 

use diagnostics;
use strict;
#require v5.5;

use File::Basename;
#fileparse_set_fstype();

use locale;
use POSIX qw(locale_h ceil floor);
setlocale(LC_ALL, "norwegian");

#to find required packages:
use FindBin; 
use lib $FindBin::Bin;
#or set
#use lib '/hf/omilia/site/home/erikve/spartan';

use SUtils qw(&slurp_sentence &del_asterisks);
use SPatterns qw($OSLO_BERGEN_V $SENTENCE_DELIM $CONT_WORD_POS_PAT $WORD_FORM_B_PAT $HEADER);



use Getopt::Std;
my %opts;
getopt('ni', \%opts);


$OSLO_BERGEN_V = ($opts{o}) ? 0 : 1; 
unless($OSLO_BERGEN_V) { $SENTENCE_DELIM = qr/CLB.*?\@\?\?\?/; }





my $S_ID = ($opts{i} || 0);
my $out = *STDOUT;  #default, overide if $opts{t}
#$out is global, and refered to by print_context_window
my $span = ($opts{n} || 3); 

 

while (@ARGV) {
    $_ = shift; #next file or dir argument
    my $file = $_; ##DEBUGGING
#    my $this_file = $_; ##for DEBUGging, see below.#
    if (-d $_) { #dips down in found directory arg:
	opendir(DIR, $_) || die "drainbamage: $! : $_\n";
	my $dir_name = $_; #saves away path, to glue it back on later.
	my @files = map { "$dir_name/$_" } grep !/(^\.|\#)|\#$|~$/, readdir(DIR);
	#glues the directory path onto files. sifts away files of form:  .foo and foo~ and #foo#
	closedir DIR || die "$!\n";
	unshift(@ARGV, @files);
 	next;  #prepends the files in the directory to the (possibly empty) list of remaining files.
    }

    my ($base,$path,$syn_suf) = &File::Basename::fileparse($file, '\.syn');
    next if (! $syn_suf);

    unless($opts{t}) {
	$path .= $base . '.wic';
	open(OFH,"> $path") || die "$!: $path";
	$out = *OFH;
    }

    slide_context_window($file, $span);
}





#     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#                   SLIDING WINDOW CONTEXT
#     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=




sub print_context_window  {
    my $buf = shift;
    my $fs = shift;
    if (! defined($fs)) { $fs = ($#{@$buf} % 2) ? (($#{@$buf} + 1) / 2) : ($#{@$buf}  / 2) };
#   if last index is even then modulo is 0 and focus is last index / 2 
  
#   for every word in focus sentence
    for (my $fw = 0; $#{@$buf[$fs]} >= $fw; $fw++) {
	print $out "${@$buf[$fs]}[$fw]\t";
#       print word followed by surrounding  context 	
	for (my $si = 0; $#{@$buf} >= $si; $si++) {
	    if($si != $fs) {  print $out " @{@$buf[$si]}";}
#           but do not print the focus word itself 
	    else {
		my @s = @{@$buf[$si]};
	      SWITCH:
		{ ($fw == 0) && do { print $out " @s[($fw + 1) .. $#s]"; last SWITCH; };
		  ($fw == $#s) && do { print $out " @s[0 .. ($fw - 1)]"; last SWITCH; };
		  do {print $out " @s[0 .. ($fw - 1)] @s[($fw + 1) .. $#s]"; last SWITCH; };
	      }
	    }
	}
	print $out "\t$S_ID\n";
    }
}






sub content_words {
    my ($buf, $pos, $end) = @_;
    return if (! $buf);
    ($pos) || ($pos = 0);
    ($end) || ($end = $#{@$buf});
    my %entry_contents;
    my @sentence_contents = ();
    while($pos++ != $end) {
	if ($$buf[$pos] =~ m/$CONT_WORD_POS_PAT/) {
	    $entry_contents{$1}++ if defined($1);
	    until(($pos >= $end) ||  #safety
		  ($$buf[++$pos] =~ m/$WORD_FORM_B_PAT/)) {
		if(($$buf[$pos] =~ m/$CONT_WORD_POS_PAT/) && 
		   (defined($1))) {
		    $entry_contents{$1}++;
		}
	    }#purge done
		@sentence_contents = (@sentence_contents, keys(%entry_contents));
	    undef %entry_contents;
	}
    }
    del_asterisks(@sentence_contents);
    return \@sentence_contents;
}





#incrementation of sentence number / $S_ID is not done directly by slurp_sentence, 
#but done separately since the relevant id is dependent on the focus word 
#in current context, not the sentences allready read. 

sub slide_context_window {
    my $filepath = shift; 
    my $span = (($_ = shift) || 3); #window span default =  3 = 1 rigth + focus + 1 left
    #note; also when supplied window width argument is zero, the default is used.
    my $break_on_headers = (($_ = shift) || 1);
    open(IN, $filepath) or die "drainbamage: $!\n";
    my $sent;      #sentence buffer
    my @win;       #context window buffer
    my $contents;  #content words
    my $foc;       #window index of focus sentence 
    my $dangling_header;
    

    do {

    #initialization 
	$#win = -1; #reset window buffer
	$foc = 0;   #reset focus index

    
      INIT:
	while(@win < $span) {
	    #get new content buffer or use dangler.
	    if (! $dangling_header) { 
		($sent = slurp_sentence(\*IN)) || last INIT;       #D BUG   
		$contents = content_words($sent);
	    } else { $dangling_header = 0; }
	    #fill up window buffer. report in accordance with $span and $foc.
	    push(@win, $contents);
	    if(@win >= ceil($span / 2))  { 
		++$S_ID; 
		print_context_window(\@win, $foc); 
		$foc++ } 
	}
	
	#slide window  
      SLIDE:
	while($sent = slurp_sentence(\*IN)) {
#	    print "$sent\n"; #DBG
	    $contents = content_words($sent); 
	    #purge window buffer if headline: 
	    if(($break_on_headers)
	       && ( $$sent[$#{@$sent}] =~ m/$HEADER/)) {
		#$content is bound to headline; 
		$dangling_header = 1; 
		last SLIDE;
	    }
	    #shift out previous sentence, push in new, report.  
	    shift(@win); 
	    push(@win, $contents);
	    ++$S_ID;
	    print_context_window(\@win);
	}
	
#      PURGE:
	#find focus word;
	$foc = floor($span / 2);
	#flush buffer, with $foc in accordance with $span    
	while(@win > ($foc + 1)) {
	    shift(@win);
	    ++$S_ID;
	    print_context_window(\@win, $foc);
	}
	
    } while($dangling_header);   #closes outer loop 'do {} while()'
 
   close(IN);
}



print "$S_ID \n";

